
// Project: webCDwriter - Network CD Writing
//
// Compare a directory or an image file to the contents of a CD
//
// Author: Jrg Haeger, 28.02.2001

#include <sys/types.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include "Error.h"
#include "Version.h"

class Verify
{
	char *buf1, *buf2;
	int bytesCounter, bytesMax;
	int checkTooLong;
	int MBytes0;
	int newLine;

public:
	Verify()
	{
		bytesMax = 1024 * 1024;
		buf1 = new char[bytesMax];
		buf2 = new char[bytesMax];
		bytesCounter = 0;
		checkTooLong = 1;
		MBytes0 = 0;
		newLine = 1;
	}

	~Verify()
	{
		delete[] buf1;
		delete[] buf2;
	}

	int compare(const char *orig, const char *copy);
	int compareFile(const char *orig, const char *copy);
	int compareTracks(const char *orig, const char *copy);
};

int Verify::compare(const char *orig, const char *copy)
{
	struct stat statBuf;
	if (stat(orig, &statBuf) != 0)
		return 0;

	if (!S_ISDIR(statBuf.st_mode))
		return compareFile(orig, copy);

	DIR *dir = opendir(orig);
	if (dir == NULL)
		return -1;

	struct dirent *dirEntry;
	while ((dirEntry = readdir(dir)) != NULL)
	{
		if (strcmp(dirEntry->d_name, ".") == 0
			|| strcmp(dirEntry->d_name, "..") == 0)
			continue;

		char tmpOrig[strlen(orig) + 1
			+ strlen(dirEntry->d_name) + 1];
		sprintf(tmpOrig, "%s/%s",
			orig, dirEntry->d_name);

		char tmpCopy[strlen(copy) + 1
			+ strlen(dirEntry->d_name) + 1];
		sprintf(tmpCopy, "%s/%s",
			copy, dirEntry->d_name);

		compare(tmpOrig, tmpCopy);
	}

	closedir(dir);
}

int Verify::compareFile(const char *orig, const char *copy)
{
	FILE *in1 = NULL, *in2 = NULL;

	try {
		in1 = fopen(orig, "r");
		if (in1 == NULL)
			throw new Error("Cannot open %s", orig);

		in2 = fopen(copy, "r");
		if (in2 == NULL)
			throw new Error("Cannot open %s", copy);

		for (int i = 0; ; )
		{
			int MBytes = bytesCounter / 1024 / 1024;
			if (MBytes > MBytes0)
			{
				if (!newLine)
					printf("\r");
				printf("%d MB", MBytes);
				fflush(stdout);
				MBytes0 = MBytes;
				newLine = 0;
			}

			int n = fread(buf1, 1, bytesMax, in1);
			if (n == 0)
				if (checkTooLong
					&& fread(buf2, 1, 1, in2) > 0)
					throw new Error("Too long");
				else
					break;

			int m = fread(buf2, 1, n, in2);

			if (memcmp(buf1, buf2, m) != 0)
				throw new Error("%s and %s differ",
					orig, copy);

			if (m < n)
				throw new Error("Too short");

			bytesCounter += n;
		}
	}
	catch (Error *err)
	{
		if (!newLine)
			printf("\n");
		printf("error: %s\n", err->toString());
		delete err;
	}

	if (in2 != NULL)
		fclose(in2);
	if (in1 != NULL)
		fclose(in1);

	return 0;
}

int Verify::compareTracks(const char *orig, const char *copy)
{
	checkTooLong = 0;
	compareFile(orig, copy);
}

int main(int argc, char *args[])
{
	printf("\nCDWverify %s (part of %s)\n", version, project);
	fflush(stdout);

	if (argc != 3)
	{
		printf("%s compares a directory "
			"or an image file to the contents of a CD\n\n"
			"Usage:\n"
			"\t\t%s directory rawDevice\n"
			"\t\t%s imageFile rawDevice\n\n",
			args[0], args[0], args[0]);
		return 1;
	}

	struct stat statBuf;
	if (stat(args[1], &statBuf) != 0)
	{
		printf("Cannot stat %s\n", args[1]);
		return 1;
	}

	Verify verify;
	if (S_ISDIR(statBuf.st_mode))
	{
		char dir[1024];
		sprintf(dir, "/tmp/cdrom");
		mkdir(dir, S_IRWXU);
		setuid(0);
		char cmdStr[1024];
		sprintf(cmdStr, "mount -r -t iso9660 %s %s 2> /dev/null",
			args[2], dir);
		if (system(cmdStr) != 0)
		{
			printf("Cannot mount the CD\n");
			return 1;
		}
		int result = verify.compare(args[1], dir);
		sprintf(cmdStr, "umount %s", dir);
		system(cmdStr);
		printf("\n");
		return result;
	}
	else
	{
		int result = verify.compareTracks(args[1], args[2]);
		printf("\n");
		return result;
	}
}
